use itertools::Itertools;

fn solve_part1(input: &str) -> i32 {
    input
        .lines()
        .filter_map(|s| s.parse().ok())
        .tuple_windows::<(i32, i32)>()
        .fold(0, |a, (l, r)| a + if l < r { 1 } else { 0 })
}

fn solve_part2(input: &str) -> i32 {
    input
        .lines()
        .filter_map(|s| s.parse().ok())
        .tuple_windows::<(i32, i32, i32)>()
        .map(|(l, m, r)| l + m + r)
        .tuple_windows()
        .fold(0, |a, (l, r)| a + if l < r { 1 } else { 0 })
}

pub fn run(input: String) {
    println!("--- Part 1 ---");
    println!("Answer: {}", solve_part1(&input));
    println!();
    println!("--- Part 2 ---");
    println!("Answer: {}", solve_part2(&input))
}

#[cfg(test)]
mod tests {
    const EXAMPLE_INPUT: &str = "199\n200\n208\n210\n200\n207\n240\n269\n260\n263";

    #[test]
    fn part1() {
        assert_eq!(super::solve_part1(EXAMPLE_INPUT), 7);
    }

    #[test]
    fn part2() {
        assert_eq!(super::solve_part2(EXAMPLE_INPUT), 5);
    }
}
